#!/bin/bash
# snat_addip.sh
# Simple shell daemon (!) that monitors connectivity to the net
# and assigns the virtual IP address if needed
# This implements failover for the VIP binding
# (c) Kurt Garloff <tsystems@garloff.de>, 2/2017
# Copyright: CC-BY-SA

# Probing parameters
# We ping google, fallback to DT routers
INETPING=8.8.8.8	# dns.google.com
INETPING2=62.154.49.126	# h-ea9-i.H.DE.NET.DTAG.DE
INETPING3=72.14.194.156	# m-ef1-i.M.DE.NET.DTAG.DE  - not used
# If we are passive (VIP not bound to us), we test VIP avail ev. INTFREQ s
INTFREQ=8
# If we are active (VIP bound to us), we ping google every EXTFREQ s
EXTFREQ=40

DEV=$1
VIP=$2
FIP=$3

if test -z "$2"; then echo "ERROR: Pass DEV VIP [FIP]" 1>&2; exit 1; fi

if test -e ~/.ostackrc; then source ~/.ostackrc; fi
if [[ "$OS_USERNAME" = *XXXXXX* ]]; then unset OS_USERNAME; fi

usleep $RANDOM
echo "Startup snat_addip.sh $DEV $VIP" | logger -t snat_addip

declare -i ctr=0

listid() { grep $1 | tail -n1 | sed 's/^| \([0-9a-f-]*\) .*$/\1/'; }

MYPORT=""
getownport()
{
  MYUUID=$(basename $(readlink /var/lib/cloud/instance))
  MYMAC=$(ip link | grep 'link/ether' | tail -n1 | sed 's@^ *link/ether \([0-9a-f:]*\).*$@\1@')
  MYPORT=$(neutron port-list | listid $MYMAC)
  test -n "$MYPORT" && echo "$MYPORT"
}

maybindfip()
{
  if test -z "$FIP" -o -z "$OS_USERNAME"; then return 0; fi
  if test -z "$MYPORT"; then return 1; fi
  # Associate FIP to it
  neutron floatingip-associate $FIP $MYPORT
}

mayunbindfip()
{
  if test -z "$FIP" -o -z "$OS_USERNAME"; then return 0; fi
  if test -z "$MYPORT"; then return 1; fi
  # Disassociate FIP from it
  neutron floatingip-disassociate $FIP #$MYPORT
}

if test -n "$FIP" -a -n "$OS_USERNAME"; then MYPORT=$(getownport); fi

while true; do
  if ip addr show $DEV | grep $VIP >/dev/null 2>&1; then
    ping -w2 -c1 $INETPING >/dev/null 2>&1
    if test $? != 0; then sleep 2; ping -w3 -c1 $INETPING2; fi
    # Release VIP
    if test $? != 0; then
      mayunbindfip
      ip addr del $VIP/32 dev $DEV
      echo "Unbinding virtual IP address $VIP from $DEV" | logger -t snat_addip
    else
      let ctr+=1
      # Avoid active active; this will bring down SNAT for one s :-( every 3 hours
      if test $(($ctr%256)) = 0; then
        id addr del $VIP/32 dev $DEV
        ping -w1 -i0.4 -c1 $VIP >/dev/null 2>&1
        if test $? != 0; then ip addr add $VIP/32 dev $DEV; fi
      fi
    fi
    sleep $EXTFREQ
  else
    ping -w2 -c1 $VIP >/dev/null 2>&1
    # Grab VIP
    if test $? != 0; then
      ip addr add $VIP/32 dev $DEV
      echo "Binding virtual IP adddress $VIP to $DEV" | logger -t snat_addip
      ctr=0
      maybindfip
    fi
    sleep $INTFREQ
  fi
  # Fuzzy logic
  usleep $((RANDOM*128))
done
